/*
Code written by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
 */
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
#endregion

namespace TopDownCharacterMovement
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class FrameSequence : Microsoft.Xna.Framework.GameComponent
    {
        private int currentFrame = -1;
        private List<FrameSequenceItem> frames = new List<FrameSequenceItem>();
        private TimeSpan lastTime;

        public void Add(FrameSequenceItem Item)
        {
            if (Item == null) throw new ArgumentNullException("Item");
            frames.Add(Item);
            if (currentFrame == -1) currentFrame = 0;
        }

        public int Count
        {
            get { return frames.Count; }
        }

        public void Remove(int Index)
        {
            frames.RemoveAt(Index);
        }

        public void Remove(FrameSequenceItem Item)
        {
            if (Item == null) throw new ArgumentNullException("Item");
            frames.Remove(Item);
        }

        public void Insert(int Index, FrameSequenceItem Item)
        {
            if (Item == null) throw new ArgumentNullException("Item");
            frames.Insert(Index, Item);
        }

        public FrameSequenceItem GetFrame(int index)
        {
            return frames[index];
        }

        public void SetItem(int index, FrameSequenceItem value)
        {
            if (value == null) throw new ArgumentNullException("value");
            frames[index] = value;
        }

        public int CurrentFrame
        {
            get { return currentFrame; }
            set { currentFrame = value; }
        }

        public FrameSequenceItem GetCurrentFrame()
        {
            if (currentFrame == -1) return null;
            return frames[currentFrame];
        }

        public FrameSequence(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
        }

        public void Reset()
        {
            currentFrame = 0;
            if (frames.Count == 0) currentFrame = -1;
            lastTime = TimeSpan.Zero;
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here
            if (currentFrame == -1) return;
            if (gameTime.TotalGameTime.Ticks > lastTime.Ticks + frames[currentFrame].DisplayTime.Ticks)
            {
                currentFrame += 1;
                if (currentFrame > frames.Count - 1) currentFrame = 0;
                if (frames.Count == 0) currentFrame = -1;
                lastTime = gameTime.TotalGameTime;
            }

            base.Update(gameTime);
        }
    }
}


