/*
Code written by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
 */
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace TopDownCharacterMovement
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        Player p1;
        FrameSequenceManager fsmanager;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            fsmanager = new FrameSequenceManager();
            this.IsMouseVisible = true;

            p1 = new Player();
            p1.Actor = new Actor(this);
            p1.GamePad = PlayerIndex.One;
            p1.Actor.Speed = 0.2f;
            p1.Actor.Position = new Vector2(this.graphics.GraphicsDevice.Viewport.Width / 2, this.graphics.GraphicsDevice.Viewport.Height / 2);
            this.Components.Add(p1.Actor);

            // this.Components.Add(new FPSCounter(this));
            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
                // create new frame sequence to store actor frames
                FrameSequence fs = new FrameSequence(this);
                for (int idx = 120; idx <= 123; idx++)
                {
                    FrameSequenceItem Item;
                    Item = new FrameSequenceItem();
                    Item.DisplayTime = TimeSpan.FromMilliseconds(100);
                    Item.Texture = content.Load<Texture2D>(@".\Images\5-" + idx.ToString());
                    Item.Center = new Vector2(14, 26);
                    fs.Add(Item);
                }

                // add walking sequence
                this.Components.Add(fs);
                fsmanager.Sequences.Add("walking", fs);

                // create standing sequence
                fs = new FrameSequence(this);
                fs.Add(fsmanager.Sequences["walking"].GetFrame(1));
                this.Components.Add(fs);
                // add a standing sequence
                fsmanager.Sequences.Add("standing", fs);
                // assign standing sequence to player's actor reference
                p1.Actor.Frames = fs;

            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {

            GamePadState state = GamePad.GetState(p1.GamePad, GamePadDeadZone.Circular);

            // get actor reference
            Actor act = p1.Actor;

            // chis if game pad is connected and if it is use it
            if (state.IsConnected == true)
            {
                // Allows the default game to exit on Xbox 360 and Windows
                if (state.Buttons.Back == ButtonState.Pressed) this.Exit();


                // move player
                Vector2 v = new Vector2(state.ThumbSticks.Left.X, -state.ThumbSticks.Left.Y) * (act.Speed * gameTime.ElapsedGameTime.Milliseconds);
                act.Position += v;
                if (v == Vector2.Zero)
                {
                    act.Frames = fsmanager.Sequences["standing"];
                }
                else
                {
                    act.Frames = fsmanager.Sequences["walking"];
                }

                // set the actors direction
                if (state.ThumbSticks.Right != Vector2.Zero)
                {
                    v = state.ThumbSticks.Right;
                    v.Y = -v.Y;
                    v.Normalize();
                    float dir = Helpers.GetRotationInRadians(0, 0, v.X, v.Y);
                    act.Direction = dir + MathHelper.ToRadians(90);
                }
                
            }
            else   // otherwire use keyboard input
            {
                HandleKeyboardInput(act, false);
                HandleMouseInput(act);
            }


            base.Update(gameTime);
        }

        private void HandleMouseInput(Actor act)
        {
            MouseState mouse = Mouse.GetState();
            float dir = Helpers.GetRotationInRadians(act.Position.X, act.Position.Y, mouse.X, mouse.Y);
            act.Direction = dir + MathHelper.ToRadians(90);
        }

        private void HandleKeyboardInput(Actor act, bool setDirection)
        {
            KeyboardState kb = Keyboard.GetState();
            if (kb.IsKeyDown(Keys.W) && kb.IsKeyDown(Keys.D))
            {
                // move player up and to the right
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(45);
                //act.Position += (Vector3.Down + Vector3.Right) * act.Speed;
                act.StrafeRight();
                act.MoveForward();

            }
            else if (kb.IsKeyDown(Keys.W) && kb.IsKeyDown(Keys.A))
            {
                // move player up and to the left
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(315);
                //act.Position += (Vector3.Down + Vector3.Left) * act.Speed;
                act.StrafeLeft();
                act.MoveForward();

            }
            else if (kb.IsKeyDown(Keys.S) && kb.IsKeyDown(Keys.A))
            {
                // move player down and to the left
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(225);
                //act.Position += (Vector3.Up + Vector3.Left) * act.Speed;
                act.StrafeLeft();
                act.MoveBackward();

            }
            else if (kb.IsKeyDown(Keys.S) && kb.IsKeyDown(Keys.D))
            {
                // move player down and to the right
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(135);
                //act.Position += (Vector3.Up + Vector3.Right) * act.Speed;
                act.StrafeRight();
                act.MoveBackward();

            }
            else if (kb.IsKeyDown(Keys.W))
            {
                // move player up
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(0);
                //act.Position += Vector3.Down * act.Speed;
                act.MoveForward();

            }
            else if (kb.IsKeyDown(Keys.A))
            {
                // move player left
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(270);
                //act.Position += Vector3.Left * act.Speed;
                act.StrafeLeft();

            }
            else if (kb.IsKeyDown(Keys.S))
            {
                // move player down
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(180);
                //act.Position += Vector3.Up * act.Speed;
                act.MoveBackward();

            }
            else if (kb.IsKeyDown(Keys.D))
            {
                // move player right
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(90);
                //act.Position += Vector3.Right * act.Speed;
                act.StrafeRight();

            }
            else if (kb.IsKeyDown(Keys.Escape))
            {
                this.Exit();

            }
            else
            {
                act.Frames = fsmanager.Sequences["standing"];
            }
        }   

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}